# flake8: noqa
__title__ = 'smsc'
__description__ = 'SMSC.ru HTTP API Library.'
__url__ = 'https://github.com/Otetz/smsc/'
__version__ = '0.1.0'
__author__ = 'Alexey Shevchenko'
__author_email__ = 'otetz@me.com'
__license__ = 'MIT'
